package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestMOV {

  private final String MNEMONIC = "MOV";
  private final byte BA = 0x51;
  private final byte CA = 0x52;
  private final byte DA = 0x53;
  private final byte EA = 0x5d;
  private final byte FA = 0x5e;

  private final byte AB = 0x61;
  private final byte AC = 0x62;
  private final byte AD = 0x63;
  private final byte AE = 0x6d;
  private final byte AF = 0x6e;

  @Test
  void testMnemonic() throws SyntaxError {
    MOV mno = new MOV(MNEMONIC + " A, B");
    mno.checkArgument();
    assertEquals(AB, mno.getByte());

    mno = new MOV(MNEMONIC + " A, C");
    mno.checkArgument();
    assertEquals(AC, mno.getByte());

    mno = new MOV(MNEMONIC + " A, D");
    mno.checkArgument();
    assertEquals(AD, mno.getByte());

    mno = new MOV(MNEMONIC + " A, E");
    mno.checkArgument();
    assertEquals(AE, mno.getByte());

    mno = new MOV(MNEMONIC + " A, F");
    mno.checkArgument();
    assertEquals(AF, mno.getByte());

    mno = new MOV(MNEMONIC + " B, A");
    mno.checkArgument();
    assertEquals(BA, mno.getByte());

    mno = new MOV(MNEMONIC + " C, A");
    mno.checkArgument();
    assertEquals(CA, mno.getByte());

    mno = new MOV(MNEMONIC + " D, A");
    mno.checkArgument();
    assertEquals(DA, mno.getByte());

    mno = new MOV(MNEMONIC + " E, A");
    mno.checkArgument();
    assertEquals(EA, mno.getByte());

    mno = new MOV(MNEMONIC + " F, A");
    mno.checkArgument();
    assertEquals(FA, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DIN mno1 = new DIN(MNEMONIC + " 0x05");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DIN mno1 = new DIN(MNEMONIC + " 0x10");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      DIN mno1 = new DIN(MNEMONIC + " A, G");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " A, B", 0);
    assertEquals(MOV.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " AB", 0);
    });

  }
}
